@@ -1,4 +1,4 @@
-#     Copyright 2024, Kay Hayen, mailto:kay.hayen@gmail.com find license text at end of file
+#     Copyright 2025, Kay Hayen, mailto:kay.hayen@gmail.com find license text at end of file
 
 
 """ Command line options of Nuitka.
@@ -43,11 +43,34 @@ if _nuitka_binary_name == "__main__.py":
 is_nuitka_run = _nuitka_binary_name.lower().endswith("-run")
 
 if not is_nuitka_run:
-    usage_template = "usage: %s [--module] [--run] [options] main_module.py"
+    usage_template = (
+        "usage: %s [--mode=compilation_mode] [--run] [options] main_module.py"
+    )
 else:
-    usage_template = "usage: %s [options] main_module.py"
+    usage_template = "usage: %s [--mode=compilation_mode] [options] main_module.py"
 
 
+def _handleHelpModes():
+    result = False
+    for count, arg in enumerate(sys.argv[1:], start=1):
+        if arg == "--":
+            break
+        if arg in ("--help-all", "--help-plugin", "--help-plugins"):
+            result = True
+            sys.argv[count] = "--help"
+            break
+    return result
+
+
+plugin_help_mode = _handleHelpModes()
+
+
+if not plugin_help_mode:
+    usage_template += """\n
+    Note: For general plugin help (they often have their own
+    command line options too), consider the output of
+    '--help-plugins'."""
+
 parser = makeOptionsParser(usage=usage_template % _nuitka_binary_name)
 
 parser.add_option(
@@ -65,6 +88,7 @@ parser.add_option(
     action="store_true",
     dest="module_mode",
     default=False,
+    github_action=False,
     help="""\
 Create an importable binary extension module executable instead of a program. Defaults to off.""",
 )
@@ -74,14 +98,18 @@ parser.add_option(
     action="store",
     dest="compilation_mode",
     metavar="COMPILATION_MODE",
-    choices=("app", "onefile", "standalone", "accelerated", "module"),
+    choices=("app", "onefile", "standalone", "accelerated", "module", "package", "dll"),
     default=None,
+    github_action_default="app",
     help="""\
 Mode in which to compile. Accelerated runs in your Python
 installation and depends on it. Standalone creates a folder
 with an executable contained to run it. Onefile creates a
 single executable to deploy. App is onefile except on macOS
-where it's not to be used. Default is 'accelerated'.""",
+where it's not to be used. Module makes a module, and
+package includes also all sub-modules and sub-packages. Dll
+is currently under development and not for users yet.
+Default is 'accelerated'.""",
 )
 
 parser.add_option(
@@ -89,10 +117,11 @@ parser.add_option(
     action="store_true",
     dest="is_standalone",
     default=False,
+    github_action=False,
     help="""\
 Enable standalone mode for output. This allows you to transfer the created binary
 to other machines without it using an existing Python installation. This also
-means it will become big. It implies these option: "--follow-imports" and
+means it will become big. It implies these options: "--follow-imports" and
 "--python-flag=no_site". Defaults to off.""",
 )
 
@@ -110,7 +139,7 @@ parser.add_option(
     action="store_true",
     dest="is_onefile",
     default=False,
-    github_action_default=True,
+    github_action=False,
     help="""\
 On top of standalone mode, enable onefile mode. This means not a folder,
 but a compressed executable is created and used. Defaults to off.""",
@@ -137,7 +166,8 @@ Python executable. Currently supported: "-S" (alias "no_site"),
 "static_hashes" (do not use hash randomization), "no_warnings" (do not
 give Python run time warnings), "-O" (alias "no_asserts"), "no_docstrings"
 (do not use doc strings), "-u" (alias "unbuffered"), "isolated" (do not
-load outside code) and "-m" (package mode, compile as "package.__main__").
+load outside code), "-P" (alias "safe_path", do not used current directory
+in module search) and "-m" (package mode, compile as "package.__main__").
 Default empty.""",
 )
 
@@ -347,6 +377,22 @@ and being non-static it's removed. Use e.g. a string like
 static cache path, this will then not be removed.""",
 )
 
+onefile_group.add_option(
+    "--onefile-cache-mode",
+    action="store",
+    dest="onefile_cached_mode",
+    metavar="ONEFILE_CACHED_MODE",
+    choices=("auto", "cached", "temporary"),
+    default="auto",
+    help="""\
+This mode is inferred from your use of the spec. If it contains
+runtime dependent paths, "auto" resolves to "temporary" which
+will make sure to remove the unpacked binaries after execution,
+and cached will not remove it and see to reuse its contents
+during next execution for faster startup times.""",
+)
+
+
 onefile_group.add_option(
     "--onefile-child-grace-time",
     action="store",
@@ -381,6 +427,18 @@ with "nuitka-onefile-unpack" rather than a stream that only the onefile
 program itself unpacks. Default is off.""",
 )
 
+onefile_group.add_option(
+    "--onefile-no-dll",
+    action="store_true",
+    dest="onefile_no_dll",
+    default=False,
+    help="""\
+When creating the onefile, some platforms (Windows currently, if not
+using a cached location) default to using DLL rather than an executable
+for the Python code. This makes it use an executable in the unpacked
+files as well. Default is off.""",
+)
+
 del onefile_group
 
 data_group = parser.add_option_group("Data files")
@@ -400,13 +458,14 @@ This will only include non-DLL, non-extension modules, i.e. actual data
 files. After a ":" optionally a filename pattern can be given as
 well, selecting only matching files. Examples:
 "--include-package-data=package_name" (all files)
-"--include-package-data=package_name=*.txt" (only certain type)
-"--include-package-data=package_name=some_filename.dat (concrete file)
+"--include-package-data=package_name:*.txt" (only certain type)
+"--include-package-data=package_name:some_filename.dat (concrete file)
 Default empty.""",
 )
 
 data_group.add_option(
     "--include-data-files",
+    "--include-data-file",
     action="append",
     dest="data_files",
     metavar="DESC",
@@ -450,6 +509,7 @@ whole directory simply use 'package_name'. Default empty.""",
 
 data_group.add_option(
     "--include-onefile-external-data",
+    "--include-data-files-external",
     action="append",
     dest="data_files_external",
     metavar="PATTERN",
@@ -503,6 +563,17 @@ not always happening. This of course only makes sense for packages that are
 included in the compilation. Default empty.""",
 )
 
+metadata_group.add_option(
+    "--list-distribution-metadata",
+    action="store_true",
+    dest="list_distribution_metadata",
+    default=False,
+    require_compiling=False,
+    help="""\
+Output the list of distributions and their details for all packages. Default not done.""",
+)
+
+
 del metadata_group
 
 dll_group = parser.add_option_group("DLL files")
@@ -531,6 +602,17 @@ dll_group.add_option(
 Output the DLLs found for a given package name. Default not done.""",
 )
 
+dll_group.add_option(
+    "--list-package-exe",
+    action="store",
+    dest="list_package_exe",
+    default="",
+    require_compiling=False,
+    help="""\
+Output the EXEs found for a given package name. Default not done.""",
+)
+
+
 del dll_group
 
 warnings_group = parser.add_option_group("Control the warnings to be given by Nuitka")
@@ -603,8 +685,9 @@ execute_group.add_option(
     dest="debugger",
     default=False,
     help="""\
-Execute inside a debugger, e.g. "gdb" or "lldb" to automatically get a stack trace.
-Defaults to off.""",
+Execute inside a debugger, e.g. "gdb" or "lldb" to automatically get a stack trace. The
+debugger is automatically chosen unless specified by name with the NUITKA_DEBUGGER_CHOICE
+environment variable. Defaults to off.""",
 )
 
 del execute_group
@@ -813,6 +896,25 @@ debug_group.add_option(
     help=SUPPRESS_HELP,
 )
 
+debug_group.add_option(
+    "--no-debug-c-warnings",
+    action="store_false",
+    dest="debug_c_warnings",
+    default=None,
+    help="""\
+Disable check normally done with "--debug". The C compilation may produce
+warnings, which it often does for some packages without these being issues,
+esp. for unused values.""",
+)
+
+debug_group.add_option(
+    "--debug-c-warnings",
+    action="store_true",
+    dest="debug_c_warnings",
+    default=None,
+    help=SUPPRESS_HELP,
+)
+
 debug_group.add_option(
     "--unstripped",
     "--unstriped",
@@ -962,6 +1064,15 @@ Create graph of optimization process internals, do not use for whole programs, b
 for small test cases. Defaults to off.""",
 )
 
+development_group.add_option(
+    "--devel-generate-ming64-header",
+    action="store_true",
+    dest="generate_mingw64_header",
+    default=False,
+    require_compiling=False,
+    github_action=False,
+    help=SUPPRESS_HELP,
+)
 
 del development_group
 
@@ -1004,7 +1115,7 @@ c_compiler_group.add_option(
     "--mingw64",
     action="store_true",
     dest="mingw64",
-    default=False,
+    default=None,
     help="""\
 Enforce the use of MinGW64 on Windows. Defaults to off unless MSYS2 with MinGW Python is used.""",
 )
@@ -1412,8 +1523,7 @@ console window unless the program was started from one. With 'disable'
 it doesn't create or use a console at all. With 'attach' an existing
 console will be used for outputs. With 'hide' a newly spawned console
 will be hidden and an already existing console will behave like
-'force'. Default is 'force'.
-""",
+'force'. Default is 'force'.""",
 )
 
 windows_group.add_option(
@@ -1503,7 +1613,8 @@ macos_group.add_option(
     "--macos-create-app-bundle",
     action="store_true",
     dest="macos_create_bundle",
-    default=False,
+    default=None,
+    github_action=False,
     help="""\
 When compiling for macOS, create a bundle rather than a plain binary
 application. This is the only way to unlock the disabling of console,
@@ -1563,7 +1674,7 @@ macos_group.add_option(
     dest="macos_app_mode",
     metavar="APP_MODE",
     choices=("gui", "background", "ui-element"),
-    default="gui",
+    default=None,
     help="""\
 Mode of application for the application bundle. When launching a Window, and appearing
 in Docker is desired, default value "gui" is a good fit. Without a Window ever, the
@@ -1572,12 +1683,22 @@ application is a "background" application. For UI elements that get to display l
 access to desktop when it does open a Window later.""",
 )
 
+macos_group.add_option(
+    "--macos-prohibit-multiple-instances",
+    action="store_true",
+    dest="macos_prohibit_multiple_instances",
+    default=False,
+    help="""\
+For application bundles, set the flag "LSMultipleInstancesProhibited" to prevent
+launching multiple instances of the application. Default is off.""",
+)
+
 macos_group.add_option(
     "--macos-sign-identity",
     action="store",
     dest="macos_sign_identity",
     metavar="MACOS_APP_VERSION",
-    default="ad-hoc",
+    default=None,
     help="""\
 When signing on macOS, by default an ad-hoc identify will be used, but with this
 option your get to specify another identity to use. The signing of code is now
@@ -1589,7 +1710,7 @@ macos_group.add_option(
     "--macos-sign-notarization",
     action="store_true",
     dest="macos_sign_notarization",
-    default=False,
+    default=None,
     help="""\
 When signing for notarization, using a proper TeamID identity from Apple, use
 the required runtime signing option, such that it can be accepted.""",
@@ -1838,7 +1959,9 @@ def _considerPluginOptions(logger):
         addUserPluginCommandLineOptions,
     )
 
-    addStandardPluginCommandLineOptions(parser=parser)
+    addStandardPluginCommandLineOptions(
+        parser=parser, plugin_help_mode=plugin_help_mode
+    )
 
     for arg in sys.argv[1:]:
         if arg.startswith(
@@ -1854,6 +1977,7 @@ def _considerPluginOptions(logger):
             addPluginCommandLineOptions(
                 parser=parser,
                 plugin_names=plugin_names.split(","),
+                plugin_help_mode=plugin_help_mode,
             )
 
         if arg.startswith("--user-plugin="):
@@ -1873,6 +1997,7 @@ run_time_variable_names = (
     "TIME",
     "PROGRAM",
     "PROGRAM_BASE",
+    "PROGRAM_DIR",
     "CACHE_DIR",
     "COMPANY",
     "PRODUCT",
@@ -2160,7 +2285,7 @@ def parseOptions(logger):
         if arg.startswith(("--main=", "--script-name=")):
             filename_args.append(arg.split("=", 1)[1])
 
-        if arg == "--module":
+        if arg in ("--mode=module", "--mode=module", "--module=package"):
             module_mode = True
 
         if arg[0] != "-":
@@ -2219,12 +2344,26 @@ def runSpecialCommandsFromOptions(options):
         displayDLLs(options.list_package_dlls)
         sys.exit(0)
 
+    if options.list_package_exe:
+        from nuitka.tools.scanning.DisplayPackageDLLs import displayEXEs
+
+        displayEXEs(options.list_package_exe)
+        sys.exit(0)
+
     if options.list_package_data:
         from nuitka.tools.scanning.DisplayPackageData import displayPackageData
 
         displayPackageData(options.list_package_data)
         sys.exit(0)
 
+    if options.list_distribution_metadata:
+        from nuitka.tools.scanning.DisplayDistributions import (
+            displayDistributions,
+        )
+
+        displayDistributions()
+        sys.exit(0)
+
     if options.edit_module_code:
         from nuitka.tools.general.find_module.FindModuleCode import (
             editModuleCode,
@@ -2246,6 +2385,14 @@ def runSpecialCommandsFromOptions(options):
         )
         sys.exit(0)
 
+    if options.generate_mingw64_header:
+        from nuitka.tools.general.generate_header.GenerateHeader import (
+            generateHeader,
+        )
+
+        generateHeader()
+        sys.exit(0)
+
 
 #     Part of "Nuitka", an optimizing Python compiler that is compatible and
 #     integrates with CPython, but also works on its own.
